/* Autogenerated by png2wx.pl on Mon May 23 13:54:18 2011 */
#ifndef ICONS_H
#define ICONS_H 1

#include <wx/wxprec.h>
#ifndef WX_PRECOMP
#	include <wx/wx.h>
#endif

extern void initialize_images(void);

extern wxBitmap *_img_vdr_pi;
extern wxBitmap *_img_vdr_play;
extern wxBitmap *_img_vdr_record;

#endif /* ICONS_H */
